from Crypto.PublicKey import ECC

"""
Generate key pairs:
private key - scalar s_alice from <0, q-1>
public key  - EC point P_alice = s_alice*G  
 
    Alice
    openssl genpkey -algorithm EC -outform PEM -pkeyopt ec_paramgen_curve:P-256 -pkeyopt ec_param_enc:named_curve -out ECpriv_alice.pem
    openssl pkey -pubout -inform PEM -outform PEM -in ECpriv_alice.pem -out ECpub_alice.pem

    Bob
    openssl genpkey -algorithm EC -outform PEM -pkeyopt ec_paramgen_curve:P-256 -pkeyopt ec_param_enc:named_curve -out ECpriv_bob.pem
    openssl pkey -pubout -inform PEM -outform PEM -in ECpriv_bob.pem -out ECpub_bob.pem

Compute shared secret:
shared secret x coordinate of s_alice*(s_bob*G) = s_bob*(s_alice*G) = (s_alice*s_bob)*G
    openssl pkeyutl -derive -out shared_alice.bin -inkey ECpriv_alice.pem -peerkey ECpub_bob.pem
    openssl pkeyutl -derive -out shared_bob.bin -inkey ECpriv_bob.pem -peerkey ECpub_alice.pem

openssl pkey -inform PEM -text -in ECpriv_alice.pem
priv:
    04:1c:bc:eb:0e:e0:ed:c0:57:41:3a:58:30:3e:ad:
    80:e0:73:3d:99:82:d6:55:e9:5e:97:a8:d6:a0:ae:
    65:8d
pub:
    04:46:41:1a:99:da:66:b4:27:11:18:bf:0e:12:6d:
    a4:37:34:f4:b6:ba:82:7f:2e:12:18:44:ff:99:b7:
    e6:af:57:a7:25:70:f2:dc:3d:15:ae:c9:0a:cf:9c:
    bf:6c:80:e4:ed:d2:96:c5:a8:e0:51:32:aa:2a:df:
    6d:2d:32:4a:ac

openssl pkey -inform PEM -text -in ECpriv_bob.pem
priv:
    4d:fc:74:fe:56:c8:55:f0:b3:57:f5:4b:b2:3d:ae:
    09:49:89:7e:0e:21:f7:63:a7:3f:42:28:7a:53:ee:
    e2:a9
pub:
    04:b1:95:69:06:26:03:31:61:1b:e9:6f:15:a8:a2:
    3c:fa:f4:af:e2:04:45:95:f3:3c:18:bc:dd:d2:28:
    b5:b5:bc:b2:c8:e6:85:aa:14:66:29:d4:04:ef:31:
    2f:d2:ec:ba:41:71:c8:f2:e2:a5:92:b3:36:2b:72:
    7b:6d:0e:ae:34

xxd -p shared_alice.bin
c0234bcfec153ce2aefb1c7aeb34825f7d91e8a14171606a1fd6cc79f1657c6b
"""
#Alice
alice_pub = ECC.import_key(open('ECpub_alice.pem').read())
alice_priv = ECC.import_key(open('ECpriv_alice.pem').read())

print(f"Alice's public key: {alice_pub.pointQ.xy}")
print(f"Alice's private key: {alice_priv.d}")


#Bob
bob_pub = ECC.import_key(open('ECpub_bob.pem').read())
bob_priv = ECC.import_key(open('ECpriv_bob.pem').read())

print(f"Bob's public key: {bob_pub.pointQ.xy}")
print(f"Bob's private key: {bob_priv.d}")


#Bob shares public key(Point Q) with Alice
#Alice calculates shared secret
alice_secret = alice_priv.d * bob_pub.pointQ
print(f"Alice's shared secret: {alice_secret.x}")

#Alice shares public key(Point Q) with Bob
#Bob calculates shared secret
bob_secret = alice_priv.d * bob_pub.pointQ
print(f"Bob's shared secret: {bob_secret.x}")

print(bob_secret == alice_secret)
print(int(bob_secret.x).to_bytes(256//8, 'little').hex())



